@@ -9,7 +9,7 @@ from autofit.tools import phase_property
 
 from autocti import exc
 from autocti.pipeline import tagging as tag
-from autocti.charge_injection import ci_fit, ci_data, ci_hyper
+from autocti.charge_injection import ci_fit, ci_data, ci_hyper, ci_mask
 from autocti.charge_injection.plotters import ci_data_plotters
 from autocti.charge_injection.plotters import ci_fit_plotters
 from autocti.data import mask as msk
@@ -41,8 +41,10 @@ class Phase(ph.AbstractPhase):
                                      previous_variable=result.previous_variable, gaussian_tuples=result.gaussian_tuples,
                                      analysis=analysis, optimizer=self.optimizer)
 
-    def __init__(self, phase_name, phase_tag=None, phase_folders=None, optimizer_class=nl.DownhillSimplex,
+    def __init__(self, phase_name, tag_phases=True, phase_folders=None, optimizer_class=nl.DownhillSimplex,
                  mask_function=msk.Mask.empty_for_shape, columns=None, rows=None,
+                 parallel_front_edge_mask_rows=None, parallel_trails_mask_rows=None,
+                 serial_front_edge_mask_columns=None, serial_trails_mask_columns=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase in an analysis pipeline. Uses the set NonLinear optimizer to try to fit models and images passed to it.
@@ -53,15 +55,36 @@ class Phase(ph.AbstractPhase):
             The class of a NonLinear optimizer
             The side length of the subgrid
         """
-        super().__init__(phase_name=phase_name, phase_tag=phase_tag, phase_folders=phase_folders,
-                         optimizer_class=optimizer_class)
+
         self.mask_function = mask_function
         self.columns = columns
         self.rows = rows
+        self.parallel_front_edge_mask_rows = parallel_front_edge_mask_rows
+        self.parallel_trails_mask_rows = parallel_trails_mask_rows
+        self.serial_front_edge_mask_columns = serial_front_edge_mask_columns
+        self.serial_trails_mask_columns = serial_trails_mask_columns
         self.cosmic_ray_parallel_buffer = cosmic_ray_parallel_buffer
         self.cosmic_ray_serial_buffer = cosmic_ray_serial_buffer
         self.cosmic_ray_diagonal_buffer = cosmic_ray_diagonal_buffer
 
+        if tag_phases:
+
+            phase_tag = tag.phase_tag_from_phase_settings(columns=columns, rows=rows,
+                                                          parallel_front_edge_mask_rows=parallel_front_edge_mask_rows,
+                                                          parallel_trails_mask_rows=parallel_trails_mask_rows,
+                                                          serial_front_edge_mask_columns=serial_front_edge_mask_columns,
+                                                          serial_trails_mask_columns=serial_trails_mask_columns,
+                                                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
+                                                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
+                                                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
+
+        else:
+
+            phase_tag = None
+
+        super().__init__(phase_name=phase_name, phase_tag=phase_tag, phase_folders=phase_folders,
+                         optimizer_class=optimizer_class)
+
     @property
     def constant(self):
         """
@@ -116,6 +139,66 @@ class Phase(ph.AbstractPhase):
         return ci_data.MaskedCIData(image=data.image, noise_map=data.noise_map, ci_pre_cti=data.ci_pre_cti, mask=mask,
                                     ci_pattern=data.ci_pattern, ci_frame=data.ci_frame)
 
+    def masks_for_analysis_from_ci_datas(self, ci_datas):
+
+        masks = list(map(lambda data: self.mask_function(shape=data.image.shape, ci_frame=data.ci_frame), ci_datas))
+
+        cosmic_ray_masks = list(map(lambda data: msk.Mask.from_cosmic_ray_image(
+            shape=data.shape, frame_geometry=data.ci_frame.frame_geometry,
+            cosmic_ray_image=data.cosmic_ray_image,
+            cosmic_ray_parallel_buffer=self.cosmic_ray_parallel_buffer,
+            cosmic_ray_serial_buffer=self.cosmic_ray_serial_buffer,
+            cosmic_ray_diagonal_buffer=self.cosmic_ray_diagonal_buffer)
+
+        if data.cosmic_ray_image is not None else None, ci_datas))
+
+        masks = list(map(lambda mask, cosmic_ray_mask:
+                         mask + cosmic_ray_mask if cosmic_ray_mask is not None else mask,
+                         masks, cosmic_ray_masks))
+
+        if self.parallel_front_edge_mask_rows is not None:
+            parallel_front_edge_masks = list(map(lambda data:
+                                                 ci_mask.CIMask.masked_parallel_front_edge_from_ci_frame(
+                                                     shape=data.shape, ci_frame=data.ci_frame,
+                                                     rows=self.parallel_front_edge_mask_rows),
+                                                 ci_datas))
+
+            masks = list(map(lambda mask, parallel_front_edge_mask: mask + parallel_front_edge_mask,
+                             masks, parallel_front_edge_masks))
+
+        if self.parallel_trails_mask_rows is not None:
+            parallel_trails_masks = list(map(lambda data:
+                                             ci_mask.CIMask.masked_parallel_trails_from_ci_frame(shape=data.shape,
+                                                                                                 ci_frame=data.ci_frame,
+                                                                                                 rows=self.parallel_trails_mask_rows),
+                                             ci_datas))
+
+            masks = list(map(lambda mask, parallel_trails_mask: mask + parallel_trails_mask,
+                             masks, parallel_trails_masks))
+
+        if self.serial_front_edge_mask_columns is not None:
+            serial_front_edge_masks = list(map(lambda data:
+                                               ci_mask.CIMask.masked_serial_front_edge_from_ci_frame(
+                                                   shape=data.shape, ci_frame=data.ci_frame,
+                                                   columns=self.serial_front_edge_mask_columns),
+                                               ci_datas))
+
+            masks = list(map(lambda mask, serial_front_edge_mask:
+                             mask + serial_front_edge_mask,
+                             masks, serial_front_edge_masks))
+
+        if self.serial_trails_mask_columns is not None:
+            serial_trails_masks = list(map(lambda data:
+                                           ci_mask.CIMask.masked_serial_trails_from_ci_frame(
+                                               shape=data.shape, ci_frame=data.ci_frame,
+                                               columns=self.serial_trails_mask_columns),
+                                           ci_datas))
+
+            masks = list(map(lambda mask, serial_trails_mask: mask + serial_trails_mask,
+                             masks, serial_trails_masks))
+
+        return masks
+
     def make_analysis(self, ci_datas, cti_settings, results=None, pool=None):
         """
         Create an analysis object. Also calls the prior passing and image modifying functions to allow child classes to
@@ -134,20 +217,8 @@ class Phase(ph.AbstractPhase):
         analysis: Analysis
             An analysis object that the non-linear optimizer calls to determine the fit of a set of values
         """
-        masks = list(map(lambda data: self.mask_function(shape=data.image.shape,
-                                                         frame_geometry=data.ci_frame.frame_geometry), ci_datas))
 
-        cosmic_ray_masks = list(map(lambda data : msk.Mask.from_cosmic_ray_image(
-            shape=data.shape, frame_geometry=data.ci_frame.frame_geometry,
-            cosmic_ray_image=data.cosmic_ray_image,
-            cosmic_ray_parallel_buffer=self.cosmic_ray_parallel_buffer,
-            cosmic_ray_serial_buffer=self.cosmic_ray_serial_buffer,
-            cosmic_ray_diagonal_buffer=self.cosmic_ray_diagonal_buffer)
-        if data.cosmic_ray_image is not None else None, ci_datas))
-
-        masks = list(map(lambda mask, cosmic_ray_mask :
-                         mask + cosmic_ray_mask if cosmic_ray_mask is not None else mask,
-                         masks, cosmic_ray_masks))
+        masks = self.masks_for_analysis_from_ci_datas(ci_datas=ci_datas)
 
         ci_datas_fit = [self.extract_ci_data(data=data, mask=mask) for data, mask in zip(ci_datas, masks)]
         ci_datas_full = list(map(lambda data, mask:
@@ -387,29 +458,21 @@ class Phase(ph.AbstractPhase):
 
 class ParallelPhase(Phase):
 
-    parallel_species = phase_property.PhasePropertyCollection("parallel_species")
+    parallel_species = phase_property.PhaseProperty("parallel_species")
     parallel_ccd = phase_property.PhaseProperty("parallel_ccd")
 
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, parallel_species=(), parallel_ccd=None,
                  optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape, columns=None,
+                 parallel_front_edge_mask_rows=None, parallel_trails_mask_rows=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
         """
 
-        if tag_phases:
-
-            phase_tag = tag.phase_tag_from_phase_settings(columns=columns, rows=None,
-                                                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
-                                                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
-                                                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
-
-        else:
-
-            phase_tag = None
-
-        super().__init__(phase_name=phase_name, phase_tag=phase_tag, phase_folders=phase_folders,
+        super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          optimizer_class=optimizer_class, mask_function=mask_function, columns=columns, rows=None,
+                         parallel_front_edge_mask_rows=parallel_front_edge_mask_rows,
+                         parallel_trails_mask_rows=parallel_trails_mask_rows,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
@@ -439,32 +502,25 @@ class ParallelPhase(Phase):
 
 class SerialPhase(Phase):
     
-    serial_species = phase_property.PhasePropertyCollection("serial_species")
+    serial_species = phase_property.PhaseProperty("serial_species")
     serial_ccd = phase_property.PhaseProperty("serial_ccd")
 
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, serial_species=(), serial_ccd=None,
                  optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape, rows=None,
+                 serial_front_edge_mask_columns=None, serial_trails_mask_columns=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
         """
 
-        if tag_phases:
-
-            phase_tag = tag.phase_tag_from_phase_settings(columns=None, rows=rows,
-                                                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
-                                                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
-                                                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
-
-        else:
-
-            phase_tag = None
-
-        super().__init__(phase_name=phase_name, phase_tag=phase_tag, phase_folders=phase_folders,
+        super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          optimizer_class=optimizer_class, mask_function=mask_function, rows=rows,
+                         serial_front_edge_mask_columns=serial_front_edge_mask_columns,
+                         serial_trails_mask_columns=serial_trails_mask_columns,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
+
         self.serial_species = serial_species
         self.serial_ccd = serial_ccd
 
@@ -489,14 +545,16 @@ class SerialPhase(Phase):
 
 
 class ParallelSerialPhase(Phase):
-    parallel_species = phase_property.PhasePropertyCollection("parallel_species")
-    serial_species = phase_property.PhasePropertyCollection("serial_species")
+    parallel_species = phase_property.PhaseProperty("parallel_species")
+    serial_species = phase_property.PhaseProperty("serial_species")
     parallel_ccd = phase_property.PhaseProperty("parallel_ccd")
     serial_ccd = phase_property.PhaseProperty("serial_ccd")
 
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, parallel_species=(), serial_species=(),
                  parallel_ccd=None, serial_ccd=None,
                  optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape,
+                 parallel_front_edge_mask_rows=None, parallel_trails_mask_rows=None,
+                 serial_front_edge_mask_columns=None, serial_trails_mask_columns=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
@@ -507,19 +565,12 @@ class ParallelSerialPhase(Phase):
             The class of a non-linear optimizer
         """
 
-        if tag_phases:
-
-            phase_tag = tag.phase_tag_from_phase_settings(columns=None, rows=None,
-                                                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
-                                                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
-                                                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
-
-        else:
-
-            phase_tag = None
-
-        super().__init__(phase_name=phase_name, phase_tag=phase_tag, phase_folders=phase_folders,
+        super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          optimizer_class=optimizer_class, mask_function=mask_function, columns=None, rows=None,
+                         parallel_front_edge_mask_rows=parallel_front_edge_mask_rows,
+                         parallel_trails_mask_rows=parallel_trails_mask_rows,
+                         serial_front_edge_mask_columns=serial_front_edge_mask_columns,
+                         serial_trails_mask_columns=serial_trails_mask_columns,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
@@ -562,7 +613,7 @@ class HyperPhase(Phase):
     """
     Mixin for hyper phases. Extracts noise scaling maps and creates MaskedCIHyperData objects for analysis.
     """
-    hyper_noise_scalars = phase_property.PhasePropertyCollection("hyper_noise_scalars")
+    hyper_noise_scalars = phase_property.PhaseProperty("hyper_noise_scalars")
 
     def __init__(self, phase_name, phase_folders, *args, **kwargs):
         super().__init__(phase_name=phase_name, phase_folders=phase_folders, *args, **kwargs)
@@ -591,9 +642,11 @@ class HyperPhase(Phase):
         analysis: Analysis
             An analysis object that the non-linear optimizer calls to determine the fit of a set of values
         """
-        masks = list(map(lambda data: self.mask_function(shape=data.image.shape,
-                                                         frame_geometry=data.ci_frame.frame_geometry), ci_datas))
+
+        masks = self.masks_for_analysis_from_ci_datas(ci_datas=ci_datas)
+
         noise_scaling_maps = self.noise_scaling_maps_from_result(results[-1])
+
         ci_datas_fit = [self.extract_ci_hyper_data(data=data, mask=mask, noise_scaling_maps=maps) for
                         data, mask, maps in zip(ci_datas, masks, noise_scaling_maps)]
         ci_datas_full = list(map(lambda data, mask, maps:
@@ -655,6 +708,7 @@ class HyperPhase(Phase):
 class ParallelHyperPhase(ParallelPhase, HyperPhase):
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, parallel_species=(), parallel_ccd=None,
                  optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape, columns=None,
+                 parallel_front_edge_mask_rows=None, parallel_trails_mask_rows=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
@@ -668,6 +722,8 @@ class ParallelHyperPhase(ParallelPhase, HyperPhase):
         super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          parallel_species=parallel_species, parallel_ccd=parallel_ccd,
                          optimizer_class=optimizer_class, mask_function=mask_function, columns=columns,
+                         parallel_front_edge_mask_rows=parallel_front_edge_mask_rows, 
+                         parallel_trails_mask_rows=parallel_trails_mask_rows,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
@@ -689,6 +745,7 @@ class SerialHyperPhase(SerialPhase, HyperPhase):
 
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, serial_species=(), serial_ccd=None,
                  optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape, rows=None,
+                 serial_front_edge_mask_columns=None, serial_trails_mask_columns=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
@@ -697,6 +754,8 @@ class SerialHyperPhase(SerialPhase, HyperPhase):
         super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          serial_species=serial_species, serial_ccd=serial_ccd, optimizer_class=optimizer_class,
                          mask_function=mask_function, rows=rows,
+                         serial_front_edge_mask_columns=serial_front_edge_mask_columns,
+                         serial_trails_mask_columns=serial_trails_mask_columns,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
@@ -716,6 +775,8 @@ class SerialHyperPhase(SerialPhase, HyperPhase):
 class ParallelSerialHyperPhase(ParallelSerialPhase, HyperPhase):
     def __init__(self, phase_name, tag_phases=True, phase_folders=None, parallel_species=(), serial_species=(), parallel_ccd=None,
                  serial_ccd=None, optimizer_class=nl.MultiNest, mask_function=msk.Mask.empty_for_shape,
+                 parallel_front_edge_mask_rows=None, parallel_trails_mask_rows=None,
+                 serial_front_edge_mask_columns=None, serial_trails_mask_columns=None,
                  cosmic_ray_parallel_buffer=10, cosmic_ray_serial_buffer=10, cosmic_ray_diagonal_buffer=3):
         """
         A phase with a simple source/CTI model
@@ -729,6 +790,10 @@ class ParallelSerialHyperPhase(ParallelSerialPhase, HyperPhase):
         super().__init__(phase_name=phase_name, tag_phases=tag_phases, phase_folders=phase_folders,
                          parallel_species=parallel_species, serial_species=serial_species, parallel_ccd=parallel_ccd,
                          serial_ccd=serial_ccd, optimizer_class=optimizer_class, mask_function=mask_function,
+                         parallel_front_edge_mask_rows=parallel_front_edge_mask_rows,
+                         parallel_trails_mask_rows=parallel_trails_mask_rows,
+                         serial_front_edge_mask_columns=serial_front_edge_mask_columns,
+                         serial_trails_mask_columns=serial_trails_mask_columns,
                          cosmic_ray_parallel_buffer=cosmic_ray_parallel_buffer,
                          cosmic_ray_serial_buffer=cosmic_ray_serial_buffer,
                          cosmic_ray_diagonal_buffer=cosmic_ray_diagonal_buffer)
